package com.gmei.data.monitor.operator;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gmei.data.monitor.bean.TblMonitorPortraitShd;
import com.gmei.data.monitor.sink.PortraitShdMysqlSink;
import com.gmei.data.monitor.utils.DateUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.flink.api.common.functions.FilterFunction;
import org.apache.flink.api.common.functions.MapFunction;
import org.apache.flink.api.java.tuple.Tuple;
import org.apache.flink.api.java.tuple.Tuple2;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.KeyedStream;
import org.apache.flink.streaming.api.datastream.SingleOutputStreamOperator;
import org.apache.flink.streaming.api.datastream.WindowedStream;
import org.apache.flink.streaming.api.functions.timestamps.AscendingTimestampExtractor;
import org.apache.flink.streaming.api.functions.windowing.ProcessWindowFunction;
import org.apache.flink.streaming.api.windowing.time.Time;
import org.apache.flink.streaming.api.windowing.windows.TimeWindow;
import org.apache.flink.util.Collector;
import org.joda.time.format.DateTimeFormatter;
import org.joda.time.format.ISODateTimeFormat;

import java.util.Arrays;
import java.util.Date;

/**
 * @ClassName PortraitMonitorShdOperator
 * @Description: 用户画像计划打点数监控
 * @Author zhaojianwei
 * @Date 2020/3/18
 * @Version V1.0
 **/
public class PortraitMonitorShdOperator implements BaseOperator{
    private DataStream dataStream;
    private int windownSize;
    private int slideSize;
    private String outJdbcUrl;
    private int maxRetry;
    private long retryInteral;
    private int parallelism;
    public static final DateTimeFormatter dateTimeFormat = ISODateTimeFormat.dateTime();
    public static final DateTimeFormatter dateTimeNoMillisFormat = ISODateTimeFormat.dateTimeNoMillis();

    public PortraitMonitorShdOperator(DataStream dataStream, int windownSize,int slideSize,String outJdbcUrl, int maxRetry, long retryInteral,int parallelism) {
        this.dataStream = dataStream;
        this.windownSize = windownSize;
        this.slideSize = slideSize;
        this.outJdbcUrl = outJdbcUrl;
        this.maxRetry = maxRetry;
        this.retryInteral = retryInteral;
        this.parallelism = parallelism;
    }

    @Override
    public void run() {
        SingleOutputStreamOperator map01 = dataStream.filter(new FilterFunction<String>() {
                @Override
                public boolean filter(String value) throws Exception {
                    return JSON.isValid(value);
                }
            }).map(new MapFunction<String, JSONObject>() {
                @Override
                public JSONObject map(String value) throws Exception {
                    return JSONObject.parseObject(value);
                }
        });
        //map01.print();
        SingleOutputStreamOperator filter01 = map01.filter(new FilterFunction<JSONObject>() {
            @Override
            public boolean filter(JSONObject jsonObject) throws Exception {
                JSONObject sysObject = jsonObject.getJSONObject("SYS");
                JSONObject appObject = jsonObject.getJSONObject("APP");
                if(sysObject != null && appObject != null && sysObject.getString("action") != null){
                    if("/api/private_conversation/".equals(sysObject.getString("action")) && appObject.get("msg_id") != null){
                        if(sysObject.getLong("errno") == 0 && appObject.getLong("antispam") == 0 && appObject.get("msg_id") != null
                        && appObject.getInteger("is_system") == 0 && appObject.getString("referrer").equals("welfare_detail")){
                            return true;
                        }
                    }else if("/api/initiate/interest_record".equals(sysObject.getString("action")) && appObject.get("content") != null){
                        return true;
                    }else if(appObject.get("info") != null && appObject.get("event_type") != null){
                        return true;
                    }else if(appObject.get("action") != null){
                        return true;
                    }else if("/api/one_image/share/v3".equals(sysObject.getString("action")) && appObject.get("ai_labels") != null){
                        return true;
                    }else if("/gm_ai/face_app/test_skin".equals(sysObject.getString("action")) && appObject.get("ai_labels") != null){
                        return true;
                    }else{
                        return false;
                    }
                }else if(jsonObject.get("type") != null && jsonObject.get("device") != null && jsonObject.get("params") != null){
                    String[] arr = {"输入", "发现", "联想", "历史"};
                    if("do_search".equals(jsonObject.getString("type")) && Arrays.asList(arr).contains(jsonObject.getJSONObject("params").getString("input_type"))){
                        return true;
                    }else if("on_click_button".equals(jsonObject.getString("type")) && jsonObject.get("params") != null){
                        JSONObject paramObject = jsonObject.getJSONObject("params");
                        if(paramObject.get("page_name") != null && paramObject.get("button_name") != null && paramObject.get("extra_param") != null ){
                            if("page_choose_interest".equals(paramObject.getString("page_name")) && "next".equals(paramObject.getString("button_name"))){
                                return false;
                            }
                        }
                    }else if("on_click_card".equals(jsonObject.getString("type")) && jsonObject.getJSONObject("params").get("card_content_type") != null ){
                        String[] array = {"diary","user_post","qa"};
                        if(Arrays.asList(array).contains(jsonObject.getJSONObject("params").getString("card_content_type"))){
                            if(jsonObject.getJSONObject("params").get("card_id") != null){
                                return true;
                            }else{
                                return false;
                            }
                        }
                    }else if("goto_welfare_detail".equals(jsonObject.getString("type")) && jsonObject.getJSONObject("params").get("service_id") != null){
                        if(jsonObject.getJSONObject("params").get("service_id") != null){
                            return true;
                        }else{
                            return false;
                        }
                    }else if("home_click_section".equals(jsonObject.getString("type")) && jsonObject.getJSONObject("params").get("icon_name") != null){
                        if(jsonObject.getJSONObject("params").get("icon_name") != null){
                            return true;
                        }else{
                            return false;
                        }
                    }else {
                        return false;
                    }
                }
                return false;
            }
        });
        SingleOutputStreamOperator filter02 = filter01.filter(new FilterFunction<JSONObject>() {
            @Override
            public boolean filter(JSONObject jsonObject) throws Exception {
                try {
                    JSONObject sysObject = jsonObject.getJSONObject("SYS");
                    if (null != sysObject) {
                        String action = sysObject.getString("action");
                        if (null != action) {
                            if ("/api/private_conversation/".equals(action)
                                    //|| "/api/initiate/interest_record".equals(action)
                                    //|| "/api/one_image/share/v3".equals(action)
                                    //|| "/gm_ai/face_app/test_skin".equals(action)
                            ) {
                                jsonObject.put("statistics_action", action);
                                return true;
                            }
                        }
                    }
                    JSONObject appObject = jsonObject.getJSONObject("APP");
                    if (null != appObject) {
                        String eventType = appObject.getString("event_type");
                        if (null != eventType) {
                            if ("validate_order".equals(eventType)
                                    || "paid_success".equals(eventType)
                                    || "add_shopcart".equals(eventType)) {
                                jsonObject.put("statistics_action", eventType);
                                return true;
                            }
                        }
                        String appAction = appObject.getString("action");
                        if (null != appAction) {
                           // String[] edits = {"create", "update", "answer"};
                           // if (Arrays.asList(edits).contains(appAction)) {
                           //     jsonObject.put("statistics_action", appAction);
                           //     return true;
                           // }
                            String[] interact = {"like", "comment", "collect"};
                            if (Arrays.asList(interact).contains(appAction)) {
                                jsonObject.put("statistics_action", appAction);
                                return true;
                            }
                        }
                    }
                    String type = jsonObject.getString("type");
                    String device = jsonObject.getString("device");
                    if (null != type && null != device) {
                        if ("do_search".equals(type)
                                || "goto_welfare_detail".equals(type)
                                || "on_click_card".equals(type)
                                //|| "home_click_section".equals(type)
                        ) {
                            jsonObject.put("statistics_action", type);
                            return true;
                        }
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    return false;
                }
                return false;
            }
        });
        SingleOutputStreamOperator singleOutputStreamOperator = filter02.assignTimestampsAndWatermarks(new AscendingTimestampExtractor<JSONObject>() {
            @Override
            public long extractAscendingTimestamp(JSONObject jsonObject) {
                long logTime = 0;

                JSONObject sysObject = jsonObject.getJSONObject("SYS");
                if (null != sysObject) {
                    String action = sysObject.getString("action");
                    if (null != action) {
                        if ("/api/private_conversation/".equals(action)
                            //|| "/api/initiate/interest_record".equals(action)
                            //|| "/api/one_image/share/v3".equals(action)
                            //|| "/gm_ai/face_app/test_skin".equals(action)
                        ) {
                            JSONObject appObject = jsonObject.getJSONObject("APP");
                            if (null != appObject) {
                                Long time = appObject.getLong("time");
                                if (null != time && time != 0) {
                                    logTime = time;
                                }
                            }
                        }
                    }
                }
                JSONObject appObject = jsonObject.getJSONObject("APP");
                if (null != appObject) {
                    String eventType = appObject.getString("event_type");
                    if (null != eventType) {
                        if ("validate_order".equals(eventType)
                                || "paid_success".equals(eventType)
                                || "add_shopcart".equals(eventType)) {
                            String backendEventTime = jsonObject.getString("TIME");
                            if (StringUtils.isNotBlank(backendEventTime)) {
                                try {
                                    logTime = dateTimeFormat.parseMillis(backendEventTime);
                                } catch (IllegalArgumentException e) {
                                    try {
                                        logTime = dateTimeNoMillisFormat.parseMillis(backendEventTime);
                                    } catch (IllegalArgumentException e2) {
                                        e2.printStackTrace();
                                    }
                                }
                            }
                        }
                    }
                    String appAction = appObject.getString("action");
                    if (null != appAction) {
                        // String[] edits = {"create", "update", "answer"};
                        // if (Arrays.asList(edits).contains(appAction)) {
                        //     jsonObject.put("statistics_action", appAction);
                        //     return true;
                        // }
                        String[] interact = {"like", "comment", "collect"};
                        if (Arrays.asList(interact).contains(appAction)) {
                            String timestamp = appObject.getString("timestamp");
                            logTime = Math.round(Double.valueOf(timestamp) * 1000);
                        }
                    }
                }
                String type = jsonObject.getString("type");
                String device = jsonObject.getString("device");
                if (null != type && null != device) {
                    if ("do_search".equals(type)
                            || "goto_welfare_detail".equals(type)
                            || "on_click_card".equals(type)
                        //|| "home_click_section".equals(type)
                    ) {
                        String maidianEventTime = jsonObject.getString("create_at");
                        if (StringUtils.isNotBlank(maidianEventTime)) {
                            logTime = Long.valueOf(maidianEventTime) * 1000;
                        }
                    }
                }
                return logTime;
            }
        });
        //filter01.print();
        SingleOutputStreamOperator map02 = singleOutputStreamOperator.map(new MapFunction<JSONObject, Tuple2<String, JSONObject>>() {
            @Override
            public Tuple2<String, JSONObject> map(JSONObject jsonObject) throws Exception {
                String statisticsAction = jsonObject.getString("statistics_action");
                return new Tuple2<String, JSONObject>(statisticsAction, jsonObject) {
                };
            }
        });
        //map02.print();
        KeyedStream keyedStream = map02.keyBy(0);
        //keyedStream.print();
        WindowedStream windowedStream = keyedStream.timeWindow(Time.seconds(windownSize), Time.seconds(slideSize));
        SingleOutputStreamOperator process = windowedStream.process(new ProcessWindowFunction<Tuple2<String, JSONObject>, TblMonitorPortraitShd, Tuple, TimeWindow>() {
            @Override
            public void process(Tuple key, Context context, Iterable<Tuple2<String, JSONObject>> elements, Collector<TblMonitorPortraitShd> out) {
                Integer count = 0;
                Date date = new Date();
                for (Tuple2<String, JSONObject> tuple2 : elements) {
                    count++;
                }
                if(count > 0){
                    String currentTimeStr = DateUtils.getCurrentTimeStr(date);
                    TblMonitorPortraitShd tblMonitorPortraitShd = new TblMonitorPortraitShd(key.getField(0), count, currentTimeStr);
                    out.collect(tblMonitorPortraitShd);
                }
            }
        });
        process
        .addSink(new PortraitShdMysqlSink(outJdbcUrl,maxRetry,retryInteral))
        .setParallelism(parallelism);
    }
}
